<?php

namespace app\em\controller;

use app\em\model\EmInvoice;
use app\em\model\EmPayment;
use app\em\model\EmPaymentInvoice;
use app\em\model\EmPaymentInvoiceBalance;
use think\Controller;
use think\facade\Log;

class WriteOff extends Controller
{
    public function writeOffFromPayment($payment_id){

        Log::info('触发销账:{$payment_id}');
        //获取缴费单对象
        $payment = EmPayment::where('payment_id','=',$payment_id)->find();

        $invModel = new EmInvoice();
        //获取关联的账单对象
        $invs =  $invModel->alias('a')->field('a.invoice_id')
            ->join(['em_payment_invoice'=>'b'],'b.invoice_id = a.invoice_id')
            ->where('b.payment_id','=',$payment_id)
            ->where('a.payment_balance','neq',0)
            ->order('a.invoice_id asc')
            ->select();

        $invsArr = [];
        $payments_balance = 0;
        //如果存在账单关联对象
        if(!$invs->isEmpty()){
            foreach ($invs as $inv){
                $invsArr[] = $inv->invoice_id;
            }
            //查询可消的额度
            $payments_balance = $invModel->where('invoice_id','in',$invsArr)->sum('payment_balance');
        }
        if($payment->balance > $payments_balance) {
            //如果可以完全销账 则继续销账
            $invss = $invModel->field('invoice_id')
                ->where('account_id', '=', $payment->account_id)
                ->where('payment_balance', 'neq', 0)
                ->whereNotIn('invoice_id', $invsArr)
                ->order('invoice_id asc')
                ->select();
            if (!$invss->isEmpty()) {
                foreach ($invss as $inv) {
                    $invsArr[] = $inv->invoice_id;
                }
            }
        }

        $paymentIds[] = $payment->payment_id;
        //销账日志
        $logMsg = date('Y-m-d H-i-s').'paymentIds :' .implode('-',$paymentIds) .'invoiceIds :' .implode('-',$invsArr);
        Log::info($logMsg);
        //处理账户金额
        return $this->writeOff($paymentIds,$invsArr);
    }

    /**
     * 账单触发销账,invoice_id 1:n payment_id
     * @param $account_id
     * @param $invoice_id
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\ModelNotFoundException
     * @throws \think\exception\DbException
     * @return mixed
     */
    public function writeOffFromInvoice($invoice_id){
        //获取账单对象
        $invoice = EmInvoice::field('id,invoice_id,account_id,payment_amount,payment_balance')
            ->where('invoice_id','=',$invoice_id)->find();

        //查询关系表中与当前invoice_id相关联的payment_id
        $model = new EmPaymentInvoice;
        $payments = $model->alias('epi')->field('epi.payment_id')
            ->leftJoin(['em_payment'=>'ep'],'epi.payment_id = ep.payment_id')
            ->leftJoin(['em_payment_invoice_balance'=>'epib'],'epib.invoice_id=epi.invoice_id')
            ->where('epib.balance','>',0)
            ->where('epi.invoice_id','=',$invoice_id)->select();
        //如果没有找到记录 证明并没有形成对应关系
        if($payments->isEmpty()){
            //查询是否有未缴费的缴费单存在
            $payments = EmPayment::field('payment_id')->where('account_id','=',$invoice->account_id)
                ->where('balance','>',0)
                ->whereIn('status','3,4,5')
                ->order('payment_id asc')
                ->select();
            if($payments->isEmpty()) return '没有找到可销账的缴费单!';
        }
        $paymentIds = [];
        foreach ($payments as $payment){
            $paymentIds[] = $payment->payment_id;
        }
        $invsIds = [
            'invoice_id' => $invoice->invoice_id
        ];

        //销账日志
        $logMsg = date('Y-m-d H-i-s').'paymentIds :' .implode('-',$paymentIds) .'invoiceIds :' .implode('-',$invsIds);
        Log::info($logMsg);
        //开始销账
        return $this->writeOff($paymentIds,$invsIds);
    }

    /**
     * @param $account_id
     * @param $invoiceIds
     * @param $paymentIds
     * @return mixed
     */
    private function writeOff($paymentIds,$invoiceIds){

        Log::info('执行销账:');

        //获取可销账缴费单
        $payments = EmPayment::where('payment_id','in',$paymentIds)->order('payment_id asc')->select();

        //获取可销账账单
        $invs = EmInvoice::where('invoice_id','in',$invoiceIds)->order('invoice_id asc')->select();

        $result = 0;
        foreach ($payments as $key => $payment){
            if($payment->balance == 0) continue;
            foreach ($invs as $index => $inv) {
                if ($inv->payment_balance == 0) continue;

                $balanceMode = new EmPaymentInvoiceBalance();
                $balanceMode->payment_id = $payment->payment_id;
                $balanceMode->invoice_id = $inv->invoice_id;

                //如果该缴费单可以完全消除当前账单额度
                if ($payment->balance >= $inv->payment_balance) {
                    //单笔销账多少
                    $balanceMode->balance = $inv->payment_balance;
                    $payment->balance = $payment->balance - $inv->payment_balance;

                    //账单额度变化
                    $inv->payment_balance = 0;
                    //更新账户status 首字母 = 2
                    $inv->status = setInvoiceStatus($inv->status, 2);
                } else {
                    //单笔销账多少
                    $balanceMode->balance = $payment->balance;

                    //账单额度变化
                    $inv->status = setInvoiceStatus($inv->status, 1);
                    $inv->payment_balance = $inv->payment_balance - $payment->balance;
                    //缴费额度变化
                    $payment->balance = 0;

                }

                //更新账单状态
                $inv->save();

                //更新缴费状态
                $payment->save();
                //查找是否已经存在相关联的账单关系
                $model = (new EmPaymentInvoice())->where(['payment_id'=>$payment->payment_id,'invoice_id'=>$inv->invoice_id])->find();
                if(!$model){
                    $model =  new EmPaymentInvoice();
                    $model->payment_id = $payment->payment_id;
                    $model->invoice_id = $inv->invoice_id;
                    $model->save();
                }
                $result++;

                $balanceMode->isUpdate(false)->save();
//                if($inv->is_pay != 1) $this->computerAmount($inv->account_id,$dk_balance);
            }
        }
        $msg = $result == 0 ? '已审核,没有找到可销账的缴费单.' : '已审核,成功销账 '.$result.' 笔账单.';
        return $msg;
    }
}